// File: client-server.cc

#include <pthread.h>
#include <iostream>
#include <utility>
#include <vector>

// ORBit2 includes
#include "orbitcpp/orb-cpp/orbitcpp.h"

#include "ClientOrbManager.h"
#include "ServerOrbManager.h"
#include "ClientServerCombinedInterface-cpp-common.h"
#include "ClientServerCombinedInterface-impl.h"

// types
typedef std::pair<int, char**> CmdLineAgrsType;

// prototypes
void* corbaClient(void*);

// global data
LockType lock;
LockType* lockPtr = &lock;

int main (int argc, char** argv)
{
  // Initialize the lock
  pthread_mutex_init(lockPtr, NULL);

  // Package the command line arguments for passing to the Server
  CmdLineAgrsType cmdLineAgrs(argc, argv); 

  // Start the Client ORB in a thread
  pthread_t threadId;
  int status = pthread_create(&threadId, NULL, corbaClient, &cmdLineAgrs);
  if (status != 0)
  {
    // Failed to start the thread
    std::cout << "Failed to start the Server ORB thread: " << status << std::endl; 
  }

  // Lock during initialization
  int lockStatus = pthread_mutex_lock(lockPtr);

  // Check if CORBA server binding name was passed as a command line
  // parameter with the format of -n "binding name" or
  // -name "binding name"
  std::vector<std::string> v;
  std::vector<std::string>::iterator vi;


  for (int i = 0; i < argc; i++)
  {
    if (argv[i] == NULL)
    {
      // end of arguments
      break;
    }
    // save the argument strings in the vector
    v.push_back(argv[i]);
  }

  // Look for the parameter in the vector
  const std::string p1Name = "-n";
  const std::string p2Name = "-name";
  std::string name = "/ServerInterface/";
  for (vi = v.begin(); vi != v.end(); vi++)  
  {
    if ((*vi == p1Name) || (*vi == p2Name))
    {
      // found the name flag
      if ((vi + 1) == v.end())
      {
        // No string after the flag - use the default name
        break;
      }
      else if ((*(vi + 1))[0] == '-')
      {
        // Found another flag - use the default name
        break;
      }
      else
      {
        // Save the context name string
        name = *(vi + 1);
        break;
      }
    }
  }

  // Initialize the connection to the ORB server
  ServerOrbManager<NLCA::ServerInterface, NLCA::ServerInterface_var, NLCA::ClientServerCombinedInterface_impl>
    serverOrb;
  if(serverOrb.initOrb(argc, argv, name) == true)
  {
    // Release lock after completing initialization
    lockStatus = pthread_mutex_unlock(lockPtr);
    if (lockStatus)
    {
      std::cout << "Client failed to release lock: lockStatus = ";
      std::cout << lockStatus << std::endl;
      exit(1);
    }

    // Enter the ORB's event loop
    std::cout << "Run ORB" << std::endl;
    serverOrb.runOrb();
  }
  else
  {
    // Release lock after completing initialization
    lockStatus = pthread_mutex_unlock(lockPtr);
    if (lockStatus)
    {
      std::cout << "Client failed to release lock: lockStatus = ";
      std::cout << lockStatus << std::endl;
      exit(1);
    }

    std::cout<<"Failed in Orb init"<<std::endl ;
    exit(1) ;
  }

  return 0 ;

} // end main




// CORBA Client
void* corbaClient(void* argsPointer)
{
  // Lock during initialization
  int lockStatus = pthread_mutex_lock(lockPtr);

  // Set up the connection to the Client interface
  CmdLineAgrsType argsPair = *(static_cast<CmdLineAgrsType*>(argsPointer));
  int argc = argsPair.first;
  char** argv = argsPair.second;

  // Check if CORBA server binding name was passed as a command line
  // parameter with the format of -n "binding name" or
  // -name "binding name"
  std::vector<std::string> v;
  std::vector<std::string>::iterator vi;

  for (int i = 0; i < argc; i++)
  {
    if (argv[i] == NULL)
    {
      // end of arguments
      break;
    }
    // save the argument strings in the vector
    v.push_back(argv[i]);
  }

  // Look for the parameter in the vector
  const std::string p1Name = "-n";
  const std::string p2Name = "-name";
  std::string name = "/ClientInterface/";
  for (vi = v.begin(); vi != v.end(); vi++)
  {
    if ((*vi == p1Name) || (*vi == p2Name))
    {
      // found the name flag
      if ((vi + 1) == v.end())
      {
        // No string after the flag - use the default name
        break;
      }
      else if ((*(vi + 1))[0] == '-')
      {
        // Found another flag - use the default name
        break;
      }
      else
      {
        // Save the context name string
        name = *(vi + 1);
        break;
      }
    }
  }

  #ifdef DEBUG
  std::cout << "name = " << name << std::endl;
  #endif

  NLCA::ClientInterface_var ClientInterfaceClientPtr;
  ClientOrbManager<NLCA::ClientInterface, NLCA::ClientInterface_var> clientOrb;
  if (clientOrb.initOrb(argc, argv) == true)
  {
    ClientInterfaceClientPtr = clientOrb.get_clientObj(name);
    if(ClientInterfaceClientPtr == CORBA_OBJECT_NIL)
      {
        std::cout<<"Failed to get client server object"<<std::endl;
        exit(1) ;
      }
  }
  else
  {
    std::cout << "ClientServerProcess: failed to connect to the Client's server.";
    std::cout << std::endl;
    exit(1);
  }

  // Release lock after completing initialization
  lockStatus = pthread_mutex_unlock(lockPtr);
  if (lockStatus)
  {
    std::cout << "Client failed to release lock: lockStatus = ";
    std::cout << lockStatus << std::endl;
    exit(1);
  }

  // Add code to call an operation on the interface
  NLCA::MyType p;
  p.att1 = 0;
  NLCA::OpStatus setStatus;
  while (1)
  {
    // Wait for lock
    lockStatus = pthread_mutex_lock(lockPtr);
    if (lockStatus)
    {
      std::cout << "Client failed to get lock: lockStatus = ";
      std::cout << lockStatus << std::endl;
      std::cout << "\tp.att1 = " << p.att1 << std::endl;
      exit(1);
    }

    // Do the work on the interface
    setStatus = ClientInterfaceClientPtr->set(p);
    p.att1--;

    // Release lock
    lockStatus = pthread_mutex_unlock(lockPtr);
    if (lockStatus)
    {
      std::cout << "Client failed to release lock: lockStatus = ";
      std::cout << lockStatus << std::endl;
      std::cout << "\tp.att1 = " << p.att1 << std::endl;
      exit(1);
    }
  }

  return 0;

} // end corbaClient
