#ifndef NAMINGSERVICEFACADE_H_HEADER_INCLUDED_C03A8D70
#define NAMINGSERVICEFACADE_H_HEADER_INCLUDED_C03A8D70

#include <string>
#include "orbitcpp/orb-cpp/orbitcpp.h"
#include "CosNaming.h"

// This class defines the interface to a CORBA Naming Service that provides the
// IOR of a CORBA server. This class implement the Facade Pattern.
//##ModelId=3FC5360D0217
class NamingServiceFacade
{
  public:
    //##ModelId=3FC5423C03BD
    NamingServiceFacade();

    ~NamingServiceFacade();

    //##ModelId=3FC539FE0004
    // This function takes the name of a CORBA server in the from
    // of a character string, parses the string, calls the naming
    // service to search its naming graph, and returns the CORBA
    // object reference to the CORBA server. Client will use this
    // reference to invoke the operations on the IDL interface
    // that is implemented by the server.
    CORBA::Object::Object * getServerObj(const std::string& bindingName);

    // This function resolves the name of the client's server via the CORBA
    // Naming Service. If successful the member data ServerCobj and
    // ptrServerObj are updated with the client's server's IOR.
    // 
    // If pass then return value is 0, ServerCobj is valid
    // and ptrServerObj if valid.
    //##ModelId=3FC53BDF0275
    int resolveNamingServiceObj(
        // This is the CORBA Object for getting the Naming Service Object
        // and storing it as an attribute.
        CORBA::ORB_ptr orb);
    // This operation registers a CORBA Server's IOR with the Naming Service.
    // Client processes use the Naming Service to get the Server's IOR, which
    // is required for accessing the operations of a CORBA Server.
    //##ModelId=3FCCCCF10009
    bool registerWithNamingService(CORBA_Object & ServerCobj,
        // This is the name the naming service uses to bind to the object
        // reference for a CORBA server
        const std::string& bindingName);


  protected:

  private:
    // This operation parses the binding name of a CORBA server. The
    // binding name is used by the naming service to store and lookup
    // an association between the binding name and the object
    // reference to the server.
    // 
    // A naming sevice stores the associations between names and object
    // references in a naming graph. The nodes of the graph are either
    // context nodes or object nodes. Context nodes are idenitified with
    // an id value and a kind value. Object nodes are associated with a
    // CORBA server object reference.
    // 
    // The id value of a context node is mandatory while the kind value
    // is optional. The CORBA specification allows any characters to 
    // make up the string values of the id and kind, even non-printing
    // characters, however this parser is designed with the following
    // restrictions:
    // * the slash (/) and period (.) characters are used as delimiters
    //   in the input character string, and therefore, they cannot be
    //   part of an id nor kind value
    // * the characters in the id and kind values should be selected
    //   from the legal characters used to create directory and file
    //   names
    // * the number of context levels in the input string is limited
    //   by any limits on the naming service, while the parse has
    //   no limit
    // * it is suggested that the kind field not be used in naming
    //   the CORBA servers in a system
    //
    // This parser uses the following format for the input string:
    // /id1.kind1/id2.kind2/id3.kind3/...
    //
    // Each context is separated by the forward slash character (/).
    // The input string may optionally start and/or end with a forward
    // slash character. Each context level must have an id value
    // which is separated from the id value with a period (.). Since
    // a kind value is optional, if it is not present in a context
    // level the period delimitor must not be present.
    //##ModelId=3FC542C302C3
    void parseName(const std::string& bindingName);

    //##ModelId=3FC539FD0331
    CORBA_Object serverCobj;

    //##ModelId=3FCBD2F90202
    //static const int NAME_SERVER_LENGTH_MAX = 2;

    //##ModelId=3FC539FD035F
    //CosNaming_NameComponent nameComp[NAME_SERVER_LENGTH_MAX];
    CosNaming_NameComponent* nameComp;

    //##ModelId=3FC539FD037F
    CosNaming_Name serverName;

    //##ModelId=3FC539FD039E
    int contextLevels;

    //##ModelId=3FCBC553027F
    CORBA::Object_ptr namingServiceObj;

    //##ModelId=3FCD4D13029C
    CORBA_Object namingServiceCobj;//need C version for parameter to naming service
    //##ModelId=3FC539FD0350
    CORBA::Object::Object *ptrServerObj;

};



#endif /* NAMINGSERVICEFACADE_H_HEADER_INCLUDED_C03A8D70 */
